<?php

use DI\Container;
use Medoo\Medoo;
use RKA\SessionMiddleware;
use Symfony\Component\Mailer\{
    Mailer,
    Transport
};

return [
    'database' => function (Container $container) {
        try {
            return (envbool('APP_DATABASE_USE') === true) ? (
                new Medoo([
                    'type' => 'mysql',
                    'host' => env('APP_DATABASE_HOST', 'localhost'),
                    'port' => env('APP_DATABASE_PORT', 3306),
                    'database' => env('APP_DATABASE_NAME', 'angiv'),
                    'username' => env('APP_DATABASE_USER', 'username'),
                    'password' => env('APP_DATABASE_PASSWORD', 'userpassword'),
                    'prefix' => env('APP_DATABASE_PREFIX', 'ang_'),
                    'charset' => 'utf8',
                    'error' => PDO::ERRMODE_EXCEPTION,
                    //'logging' => true
                ])
            ) : null;
        } catch (\Exception $e) {
            $logger=$container->get('logger'); //$GLOBALS['app']->getContainer()->get('logger');
            $logger->error('Error connect database', ['message' => $e->getMessage()]);
            return null;
        }
    },

    'session' => function () {
        $session=new SessionMiddleware([
            'name' => 'sessiondata',
            'lifetime' => 31536000,
            'path' => '/',
            //'domain' => 'angiv',
            'httponly' => true,
            'secure' => true,
            'cache_limiter' => 'nocache'
        ]);
        $session->start();
        return $session;
    },

    'mailer' => function () {
        try {
            return ((envbool('APP_MAIL_USE') === true) && (env('APP_MAIL_USER') !== null)) ? (
                new Mailer(Transport::fromDsn(sprintf(
                    '%s://%s:%s@%s:%s',
                    env('APP_MAIL_TYPE', 'smtp'),
                    env('APP_MAIL_USER'),
                    urlencode(env('APP_MAIL_PASSWORD', 'userpassword')),
                    env('APP_MAIL_HOST', 'smtp.mailserver.org'),
                    env('APP_MAIL_PORT', '25')
                )))
            ) : null;
        } catch (Exception $e) {
            return null;
        }
    }
];