<?php

namespace Plugin\auth\src\controller;

use Slim\Psr7\{
    Request,
    Response,
    Cookies
};
use Firebase\JWT\JWT;
use App\response\customResponse;

class ajaxauthController
{
    /**
     * @route({"url": "login", "methods": ["post"]})
     */
    public static function login(Request $request, Response $response) {
        $data=$request->getParsedBody();
        if ($data === null || empty($data) || !is_array($data)) {return customResponse::abort(__('Input data error'), $request);}
        if (!isset($data['login']) || !isset($data['pass'])) {return customResponse::abort(__('No password or login is set'), $request);}
        // -----
        try {
            $res=false; $token=null;

            $authusers=[];
            foreach ($_ENV as $key => $item) {
                if (strpos($key, 'APP_AUTH_USER_') === 0) {
                    $authusers[substr($key, strlen('APP_AUTH_USER_'))]=array_map('trim', explode(',', $item));
                }
            }
            $usersdb=[];
            if (!empty($db=$GLOBALS['app']->getContainer()->get('database'))) {
                $tablename=str_replace('"', '', $db->tableQuote('users'));
                if ($db->query("SHOW TABLES LIKE '{$tablename}'")->fetchColumn() !== false) {
                    $usersdb=$db->select('users', ['login', 'password']);
                    if (!empty($usersdb) && count($usersdb) !== 0) {
                        foreach ($usersdb as $item) {
                            $authusers[$item['login']]=array($item['password']);
                        }
                    }
                }
            }

            foreach ($authusers as $key => $item) {
                if ($data['login'] == $key && !empty($item[0]) && password_verify($data['pass'], $item[0])) {
                    $now=time();
                    $exp=(int)env('APP_AUTH_EXP', 0);
                    $secret=env('APP_AUTH_SECRET', '');

                    $payload=[
                        'iat' => $now,
                        //'nbf' => $now + 10,
                        'exp' => $now + (($exp == 0) ? 31536000 : $exp),
                        'jti' => base64_encode(random_bytes(32)),
                        //'iss' => 'mydomainname',
                        //'sub' => 'myemail',
                        'data' => [
                            'authlogin' => $key
                        ]
                    ];
                    $token=JWT::encode($payload, $secret, 'HS256');

                    $cookies=new Cookies();
                    $cookies->setDefaults([
                        'expires' => ($exp == 0) ? $exp : $now + $exp,
                        'path' => '/',
                        //'domain' => 'angiv',
                        'hostonly' => false,
                        'httponly' => true,
                        'secure' => true,
                        'samesite' => 'Lax'
                    ])->set('accesstoken', $token);
                    $response=$response->withHeader('Set-Cookie', $cookies->toHeaders());
                    unset($cookies);

                    $res=true;
                }
            }

            if ($res === false) {
                if (empty($_ENV['APP_LOG_USE']) || envbool('APP_LOG_USE') === true) {
                    if (envbool('APP_LOG_AUTH') === true) {
                        $logger=$GLOBALS['app']->getContainer()->get('logger');
                        $logger->error('Authorization denied',  array_merge((array)$arguments, 
                            [
                                'data_login' => $data['login'],
                                //'data_pass' => $data['pass']
                            ]
                        ));
                    }
                }
            }

            return customResponse::json($response, ['res' => $res, 'token' => $token]);
        } catch (\Exception $e) {
            return customResponse::abort($e->getMessage(), $request);
        }
    }

    /**
     * @route({"url": "logout", "methods": ["post"]})
     */
    public static function logout(Request $request, Response $response) {
        try {
            $cookies=new Cookies();
            if (!empty($cookies->get('accesstoken')) || isset($_COOKIE['accesstoken'])) {
                $cookies->set('accesstoken', ['value' => '', 'expires' => time() - 3600, 'path' => '/']);
                $response=$response->withHeader('Set-Cookie', $cookies->toHeaders());
                unset($cookies);
                unset($_COOKIE['accesstoken']);
            }

            return customResponse::json($response, true);
        } catch (\Exception $e) {
            return customResponse::abort($e->getMessage(), $request);
        }
    }

    /*
    / **
     * @route({"url": "getjwtdata", "methods": ["post"]})
     * /
    public static function getjwtdata(Request $request, Response $response) {
        $res=['res' => true, 'token' => $request->getAttribute('jwtdata')];
        return customResponse::json($response, $res);
    }
    */
}