<?php

use Slim\{
    Psr7\Request,
    Psr7\Response,
    Views\PhpRenderer,
    Routing\RouteCollectorProxy as Group
};
use App\middleware\{
    rendererMiddleware,
    roleaccessMiddleware
};

$admingroup=$app->group($_ENV['APP_ADMIN_URL'], function (Group $group) use ($container, $pagereg, $currenturl) {
    $collection=$container->get('collection'); $plugins=$collection['plugins']; $homeurl=$collection['homeurl'];

    $phpdatacontrolpanel=[
        'plugins' => $plugins,
        'locale' => $container->get('translator')->getLocale(),
        'authuse' => envbool('APP_AUTH_USE'),
        'databaseuse' => envbool('APP_DATABASE_USE'),
        'settingsshow' => envbool('APP_ADMIN_SHOW_SETTINGS'),
        'adminurl' => $_ENV['APP_ADMIN_URL'],
        'homeurl' => $homeurl,
        'currenturl' => (!empty($currenturl) ? $currenturl : '')
    ];

    $renderer=new PhpRenderer(base_path('/angiv/plugin/'), ['phpdatacontrolpanel' => $phpdatacontrolpanel], '../../app/admin/theme/template.php');
    $container->set('viewadmin', $renderer);

    $group->get('', function (Request $request, Response $response) use ($renderer) {
        $renderer->addAttribute('css', filelist(['/app/admin/css/dashboard.css']));
        $renderer->addAttribute('js', filelist(['/app/admin/js/dashboard.js']));

        return $renderer->render($response, '../../app/admin/view/dashboard.php', ['title' => __('Control panel')]);
    })->add(new rendererMiddleware($container, 'viewadmin'));
    foreach (routelist('App\admin\src\controller\dashboardController') as $item) {
        $route=$group->map($item['methods'], str_replace('//', '/', '/' . $item['url']), $item['action']);
    }

    $group->get('/users', function (Request $request, Response $response) use ($container, $renderer) {
        $usersenv=[]; $usersdb=[];
        $usersallcontrollerpath=filelist(['src/controller/admin/pageusersController.php'], 'users');
        if (!empty($usersallcontrollerpath) && count($usersallcontrollerpath) !== 0) {require_once $usersallcontrollerpath[0];}

        $renderer->addAttribute('css', filelist(['/system/animate/css/animate.min.css', 'css/admin/pageusers.css'], 'users'));
        $renderer->addAttribute('js', filelist(['/system/animate/js/animate.js', '/system/textautoresize/textautoresize.min.js', '/system/validator/validator.min.js', 'js/admin/pageusers.js'], 'users'));

        return $renderer->render($response, 'users/view/admin/pageusers.php', ['title' => __('Users'), 'phpdata' => ['usersenv' => $usersenv, 'usersdb' => $usersdb]]);
    })->add(new rendererMiddleware($container, 'viewadmin'));
    if (str_starts_with($currenturl, '/xadmin/users') !== false) {
        foreach (routelist('Plugin\users\src\controller\admin\ajaxadminController') as $item) {
            $route=$group->map($item['methods'], '/users' . str_replace('//', '/', '/' . $item['url']), $item['action']);
        }
    }

    $group->get('/roles', function (Request $request, Response $response) use ($container, $renderer) {
        $rolesenv=[];
        foreach ($_ENV as $key => $item) {
            if (strpos($key, 'APP_PERM_ROLE_') === 0) {
                $rolesenv[substr($key, strlen('APP_PERM_ROLE_'))]=array_map('trim', explode(',', strtolower($item)));
            }
        }

        $rolesdb=[];
        if (!empty($db=$container->get('database'))) {
            $tablename=str_replace('"', '', $db->tableQuote('users'));
            if ($db->query("SHOW TABLES LIKE '{$tablename}'")->fetchColumn() !== false) {
                $usersdb=$db->select('users', ['login', 'roles']);
                if (!empty($usersdb) && count($usersdb) !== 0) {
                    foreach ($usersdb as $item) {
                        if (!empty($item['roles'])) {
                            $rolesdb=array_merge($rolesdb, array_map('trim', explode(',', strtolower($item['roles']))));
                        }
                    }
                    $rolesdb=array_unique($rolesdb);
                    sort($rolesdb, SORT_NATURAL | SORT_FLAG_CASE);
                }
                if (!empty($rolesdb) && count($rolesdb) !== 0) {
                    $rolesdb=array_fill_keys($rolesdb, []);
                    foreach ($rolesdb as $key => $role) {
                        foreach ($usersdb as $user) {
                            if (!empty($user['roles']) && in_array($key, array_map('trim', explode(',', strtolower($user['roles']))))) {
                                $rolesdb[$key][]=$user['login'];
                            }
                        }
                    }
                }
            }
        }

        $renderer->addAttribute('js', filelist(['/app/admin/js/roles.js']));

        return $renderer->render($response, '../../app/admin/view/roles.php', ['title' => __('Roles'), 'phpdata' => ['rolesenv' => $rolesenv, 'rolesdb' => $rolesdb]]);
    })->add(new rendererMiddleware($container, 'viewadmin'));

    $group->get('/plugins', function (Request $request, Response $response) use ($plugins, $renderer) {
        $listpluginspagesadmincount=[];
        if (!empty($plugins) && count($plugins) != 0) {
            foreach ($plugins as $plugin) {
                $listpluginspagesadmincount[$plugin['dir']]=$plugin['pagesadmincount'];
            }
        }
        $phpdata=[
            'pluginactiveindx' => null,
            'pluginlistpages' => [],
            'newhomeurl' => '',
            'listpluginspagesadmincount' => $listpluginspagesadmincount
        ];

        $renderer->addAttribute('js', filelist(['/app/admin/js/plugins.js']));

        return $renderer->render($response, '../../app/admin/view/plugins.php', ['title' => __('Plugins'), 'phpdata' => $phpdata]);
    })->add(new rendererMiddleware($container, 'viewadmin'));
    if (str_starts_with($currenturl, '/xadmin/plugins') !== false) {
        foreach (routelist('App\admin\src\controller\pluginController') as $item) {
            $route=$group->map($item['methods'], '/plugins' . str_replace('//', '/', '/' . $item['url']), $item['action']);
        }
    }

    if (envbool('APP_ADMIN_SHOW_SETTINGS') === true) {
        $group->get('/settings', function (Request $request, Response $response) use ($renderer) {
            $phpdata=[
                'configenv' => highlight_file(base_path('/angiv/config/config.env'), true)
            ];

            $renderer->addAttribute('css', filelist(['/app/admin/css/settings.css']));
            $renderer->addAttribute('js', filelist(['/app/admin/js/settings.js']));

            return $renderer->render($response, '../../app/admin/view/settings.php', ['title' => __('Settings'), 'phpdata' => $phpdata]);
        })->add(new rendererMiddleware($container, 'viewadmin'));
    }

    $group->get('/infos', function (Request $request, Response $response) use ($container, $renderer) {
        $info=[];
        $info['cmfversion']=$_ENV['APP_VERSION'];
        $info['server']=(!empty($_SERVER['SERVER_SOFTWARE'])) ? $_SERVER['SERVER_SOFTWARE'] : __('No data available');
        $info['useragent']=(!empty($_SERVER['HTTP_USER_AGENT'])) ? $_SERVER['HTTP_USER_AGENT'] : __('No data available');

        $info['phpversion']=phpversion();
        $info['sapiname']=php_sapi_name();
        $info['workson']=(function_exists('php_uname')) ? php_uname() : __('No data available');
        $info['extensions']=implode(', ', get_loaded_extensions());

        $info['driver']=false;
        //if (envbool('APP_DATABASE_USE') === true) {
            if (!empty($db=$container->get('database'))) {
                $info['dbversion']=$db->pdo->getAttribute(\PDO::ATTR_SERVER_VERSION);
                $info['driver']=$db->pdo->getAttribute(\PDO::ATTR_DRIVER_NAME);
                $info['client']=$db->pdo->getAttribute(\PDO::ATTR_CLIENT_VERSION);
            }
        //}

        $renderer->addAttribute('js', filelist(['/app/admin/js/infos.js']));

        return $renderer->render($response, '../../app/admin/view/infos.php', ['title' => __('Information'), 'phpdata' => $info]);
    })->add(new rendererMiddleware($container, 'viewadmin'));

    if (!empty($plugins) && count($plugins) != 0) {
        foreach ($plugins as $plugin) {
            if (($plugin['isused'] == 'on' || in_array('alwaysuse', $plugin['tags'])) && (empty($plugin['isremoved']) || $plugin['isremoved'] == 'no')) {
                if ($container->get('uri')['method'] == 'GET') {
                    foreach ($plugin['pagesadmin'] ?? [] as $key => $page) {
                        $pagereg($plugin, $page, 'viewadmin');
                    }

                    if (!empty($plugin['pagesadmin']) && count($plugin['pagesadmin']) != 0) {
                        $redirecturl=$_ENV['APP_ADMIN_URL'] . $plugin['pagesadmin'][0]['url'];
                        $group->get('/' . $plugin['dir'], function (Request $request, Response $response) use ($redirecturl) {
                            return $response->withHeader('Location', $redirecturl)->withStatus(301);
                        });
                    }
                }

                if ($container->get('uri')['method'] != 'GET') {
                    if (!empty($plugin['ajaxadmin']) && count($plugin['ajaxadmin']) != 0) {
                        foreach ($plugin['ajaxadmin'] as $ajaxroute) {
                            $ajaxdir=(($ajaxroute[0] != '/') ? $plugin['dir'] : explode('/', $ajaxroute)[1]);
                            $ajaxcontroller='Plugin' . (($ajaxroute[0] != '/') ? ("\\{$plugin['dir']}\\") : '') . str_replace('/', '\\', str_replace('.php', '', $ajaxroute));
                            foreach (routelist($ajaxcontroller) as $ajaxmethod) {
                                $ajaxroute=$group->map($ajaxmethod['methods'], "/{$ajaxdir}/{$ajaxmethod['url']}", $ajaxmethod['action']);
                                if (!empty($ajaxmethod['name'])) {$ajaxroute->setName($ajaxmethod['name']);}
                            }
                        }
                    }

                    if (!empty($plugin['scriptdatabase'])) {
                        $group->post('/' . $plugin['dir'] . '/executescriptdatabase', "Plugin\\{$plugin['dir']}\\scriptdatabase::executescriptdatabase");
                    }
                }
            }
        }

        $group->get('/sitepages[/{urlparams:.*}]', function (Request $request, Response $response, $urlparams = '') use ($container, $renderer) {
            $title=__('Site pages');

            $pagesuser=[];
            $plugininfo=@include base_path('/angiv/plugin/' . $urlparams . '/index.php');
            if (!empty($plugininfo['pagesuser']) && count($plugininfo['pagesuser']) != 0) {
                foreach ($plugininfo['pagesuser'] as $page) {
                    if (!empty($page['url']) && !empty($page['controller'])) {
                        $title=(!empty($page['title'])) ? __($page['title']) : $page['url'];
                        $url=str_replace('//', '/', '/' . $page['url']);
                        $pagesuser[]=['title' => $title, 'url' => $url];
                    }
                }
                $title=$plugininfo['name'];
            }

            $renderer->addAttribute('js', filelist(['/app/admin/js/sitepages.js']));

            return $renderer->render($response, '../../app/admin/view/sitepages.php', ['title' => $title, 'phpdata' => ['pagesuser' => $pagesuser]]);
        })->add(new rendererMiddleware($container, 'viewadmin'));
    }
})->
    add($container->get('csrf'));

if (envbool('APP_AUTH_USE') === true) {
    $admingroup->
        add(new roleaccessMiddleware($container, ['control_panel'], true))->
        add($container->get('auth'));
}